# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import importlib, pkgutil
import runpy
import sys, os

import TermTk as ttk
from tlogg import TloggPlugin

class TloggHelper():
    @staticmethod
    def _loadPlugins():
        # Check for the plugin folder
        pluginFolder = os.path.join(os.path.dirname(os.path.abspath(__file__)),'plugins')
        if not os.path.exists(pluginFolder):
            ttk.TTkLog.error("No 'plugins' folder found in the 'tlogg' main directory")
        else:
            for fn in os.listdir(pluginFolder):
                filePath = os.path.join(pluginFolder,fn)
                if not os.path.isfile(filePath): continue
                absolute_name = importlib.util.resolve_name(filePath, None)
                ttk.TTkLog.debug(absolute_name)
                loader = importlib.machinery.SourceFileLoader(fn, filePath)
                spec = importlib.util.spec_from_loader(loader.name, loader)
                mod = importlib.util.module_from_spec(spec)
                loader.exec_module(mod)

        # Check installed plugins
        for finder, name, ispkg in pkgutil.iter_modules():
            if name.startswith("tlogg_"):
                loader = importlib.find_loader(name)
                spec = importlib.util.find_spec(name)
                mod = importlib.util.module_from_spec(spec)
                # runpy.run_module(name)
                loader.exec_module(mod)

        # check the plugin folder
        for mod in TloggPlugin.instances:
            if mod.init is not None:
                mod.init()

    @staticmethod
    def _runPlugins():
        for mod in TloggPlugin.instances:
            if mod.apply is not None:
                mod.apply()

    @staticmethod
    def _getPluginPlacements():
        ret = ttk.TTkK.NONE
        for mod in TloggPlugin.instances:
            ret |= mod.position
        return ret

    @staticmethod
    def _getPlacedPlugins(placement):
        return [mod for mod in TloggPlugin.instances if mod.position & placement]




#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ['PredefinedFilters','PredefinedFiltersFormWindow']

import copy
from readline import insert_text

from . import TloggCfg, TloggGlbl

from TermTk import *

class PredefinedFiltersFormWindow(TTkWindow):
    '''
    This form is inspired by klogg "PredefinedFilters..." form
    '''

    def __init__(self, *args, **kwargs):
        TTkWindow.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'PredefinedFiltersFormWindow' )

        self._filters = copy.deepcopy(TloggCfg.filters)
        self._selected = -1

        self._retLayout     = TTkVBoxLayout() # Full Layout
        self._controlLayout = TTkGridLayout() # Layout including the add/remove/move buttons
        self._filtersLayout = TTkGridLayout() # Layout having the filters form
        self._bottomLayout  = TTkGridLayout() # Layout including the OK,Cancel,Apply Buttons

        self.setLayout(self._retLayout)

        self._retLayout.addItem(self._controlLayout)
        self._retLayout.addItem(self._filtersLayout)
        self._retLayout.addItem(self._bottomLayout)

        self._addButton    = TTkButton(text="+",maxWidth=3)
        self._removeButton = TTkButton(text="-",maxWidth=3)
        self._upButton     = TTkButton(text="▲",maxWidth=3)
        self._downButton   = TTkButton(text="▼",maxWidth=3)

        self._controlLayout.addWidget(self._addButton    ,1,0)
        self._controlLayout.addWidget(self._removeButton ,1,1)
        self._controlLayout.addWidget(TTkSpacer(maxHeight=1) ,1,2)
        self._controlLayout.addWidget(self._upButton     ,1,3)
        self._controlLayout.addWidget(self._downButton   ,1,4)

        self._bottomLayout.addWidget(applyBtn  := TTkButton(text="Apply",  border=True, maxHeight=3),0,1)
        self._bottomLayout.addWidget(cancelBtn := TTkButton(text="Cancel", border=True, maxHeight=3),0,2)
        self._bottomLayout.addWidget(okBtn     := TTkButton(text="OK",     border=True, maxHeight=3),0,3)

        self._refreshFilters()

        def _move(offset):
            def _moveUpDown():
                if self._selected < 0: return
                if self._selected + offset < 0: return
                if self._selected + offset >= len(self._filters): return
                item = self._filters.pop(self._selected)
                self._filters = self._filters[:self._selected+offset] + [item] + self._filters[self._selected+offset:]
                self._refreshFilters()
                self._selected += offset
            return _moveUpDown
        self._upButton.clicked.connect(_move(-1))
        self._downButton.clicked.connect(_move(1))

        def _addCallback():
            filter = {'name':f'n:{len(self._filters)+1}', 'pattern':'' }
            self._filters.append(filter)
            self._selected = -1
            self._refreshFilters()
        self._addButton.clicked.connect(_addCallback)

        def _removeCallback():
            if self._selected > -1:
                self._filters.pop(self._selected)
            self.selected = -1
            self._refreshFilters()
        self._removeButton.clicked.connect(_removeCallback)

        def _saveFilters():
            TloggCfg.filters = self._filters
            TloggCfg.save(searches=False, filters=True, colors=False, options=False)

        applyBtn.clicked.connect(_saveFilters)
        okBtn.clicked.connect(_saveFilters)
        okBtn.clicked.connect(self.close)
        cancelBtn.clicked.connect(self.close)

    def _refreshFilters(self):
        for item in self._filtersLayout.children():
            self._filtersLayout.removeItem(item)
        splitter = TTkSplitter(border=True)
        names    = TTkVBoxLayout()
        patterns = TTkVBoxLayout()
        splitter.addWidget(TTkFrame(border=False, layout=names))
        splitter.addWidget(TTkFrame(border=False, layout=patterns))
        splitter.setSizes([20,100])
        for i, filter in enumerate(self._filters):
            names.addWidget(   nl := TTkLineEdit(text=filter['name']))
            patterns.addWidget(pl := TTkLineEdit(text=filter['pattern']))
            # Stupid way to attach the textEdited signal directly to the array modification
            # It could have been made much better but Lazyness is an Issue
            def _setName(pos):
                def _set(txt): self._filters[pos]['name'] = txt
                return _set
            def _setPattern(pos):
                def _set(txt): self._filters[pos]['pattern'] = txt
                return _set
            def _setSelected(pos):
                def _set(focus):
                    if focus:
                        self._selected = pos
                return _set
            nl.textEdited.connect(_setName(i))
            pl.textEdited.connect(_setPattern(i))
            nl.focusChanged.connect(_setSelected(i))
            pl.focusChanged.connect(_setSelected(i))

        names.addWidget(   TTkSpacer())
        patterns.addWidget(TTkSpacer())
        self._filtersLayout.addWidget(splitter)


class PredefinedFilters(TTkResizableFrame):
    __slots__ = ('checked', 'unchecked', '_searchbox')
    def __init__(self, searchbox):
        layout=TTkVBoxLayout()
        TTkResizableFrame.__init__(self, layout=layout)
        self._name = 'PredefinedFilters'

        self._searchbox = searchbox

        filters = copy.deepcopy(TloggCfg.filters)
        for filter in filters:
            def _cb(p, sb):
                def _ret(state):
                    txt = sb.currentText()
                    if state == TTkK.Checked:
                        if not txt:
                            txt = p
                        elif txt.find(p) == -1:
                            txt += '|'+p
                    else:
                        p1 = f"|{p}"
                        p2 = f"{p}|"
                        if txt.find(p1) != -1:
                            txt = txt.replace(p1,'')
                        elif txt.find(p2) != -1:
                            txt = txt.replace(p2,'')
                        else:
                            txt = txt.replace(p,'')
                    sb.setCurrentText(txt)
                return _ret

            layout.addWidget(cb := TTkCheckbox(text=filter['name'],checked=searchbox.currentText().find(filter['pattern'])!=-1))
            cb.stateChanged.connect(_cb(filter['pattern'], searchbox))
        w,h = self.minimumSize()
        self.resize(w,h)




#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ['TloggCfg']

import os
import yaml

class TloggCfg:
    version="0.6.0a"
    name="tlogg"
    cfgVersion = '1.0'
    pathCfg="."
    colors=[]
    filters=[]
    options={}
    searches=[]
    maxsearches=200

    @staticmethod
    def save(searches=True, filters=True, colors=True, options=True):
        os.makedirs(TloggCfg.pathCfg, exist_ok=True)
        colorsPath   = os.path.join(TloggCfg.pathCfg,'colors.yaml')
        filtersPath  = os.path.join(TloggCfg.pathCfg,'filters.yaml')
        optionsPath  = os.path.join(TloggCfg.pathCfg,'options.yaml')
        searchesPath = os.path.join(TloggCfg.pathCfg,'searches.yaml')

        def writeCfg(path, cfg):
            fullCfg = {
                'version':TloggCfg.cfgVersion,
                'cfg':cfg }
            with open(path, 'w') as f:
                yaml.dump(fullCfg, f, sort_keys=False, default_flow_style=False)

        if colors:   writeCfg(colorsPath,   TloggCfg.colors)
        if filters:  writeCfg(filtersPath,  TloggCfg.filters)
        if options:  writeCfg(optionsPath,  TloggCfg.options)
        if searches: writeCfg(searchesPath, TloggCfg.searches)

    @staticmethod
    def load():
        colorsPath   = os.path.join(TloggCfg.pathCfg,'colors.yaml')
        filtersPath  = os.path.join(TloggCfg.pathCfg,'filters.yaml')
        optionsPath  = os.path.join(TloggCfg.pathCfg,'options.yaml')
        searchesPath = os.path.join(TloggCfg.pathCfg,'searches.yaml')

        if os.path.exists(colorsPath):
            with open(colorsPath) as f:
                TloggCfg.colors = yaml.load(f, Loader=yaml.SafeLoader)['cfg']
        if os.path.exists(filtersPath):
            with open(filtersPath) as f:
                TloggCfg.filters = yaml.load(f, Loader=yaml.SafeLoader)['cfg']
        if os.path.exists(optionsPath):
            with open(optionsPath) as f:
                TloggCfg.options = yaml.load(f, Loader=yaml.SafeLoader)['cfg']
        if os.path.exists(searchesPath):
            with open(searchesPath) as f:
                TloggCfg.searches = yaml.load(f, Loader=yaml.SafeLoader)['cfg']



# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import re
import sys
import argparse

import appdirs

from TermTk import *

from TermTk import TTk, TTkK, TTkLog, TTkMenuButton
from TermTk import pyTTkSlot, pyTTkSignal
from TermTk import TTkFrame

from tlogg import TloggHelper, tloggProxy

from .cfg  import *
from .glbl import *
from .about import *
from .loggwidget import LoggWidget
from .options import optionsFormLayout, optionsLoadTheme
from .highlighters import highlightersFormLayout
from .predefinedfilters import PredefinedFiltersFormWindow
from .notepad import NotePad


class TLOGG(TTkGridLayout):
    '''
        ┌──────────────────[Main Splitter]─────────┐
        │┌──────────╥────[File Tab Splitter]──────┐│
        ││ F.Tree   ║       Tab                   ││
        ││ Controls ╟─────────────────────────────┤│
        │├──────────╢┌───────────────────────────┐││
        ││ File     ║│   File Viewer             │││
        ││ Tree     ║│                           │││
        ││          ║╞═══════════════════════════╡││
        ││          ║│   Search Widget           │││
        ││          ║│                           │││
        ││          ║└───────────────────────────┘││
        │└──────────╨─────────────────────────────┘│
        ╞══════════════════════════════════════════╡
        │ Logger,Debug View                        │
        └──────────────────────────────────────────┘
    '''
    __slots__ = ('_kodeTab', '_tloggProxy','_notepad')
    def __init__(self, tloggProxy, *args, **kwargs) -> None:
        self._tloggProxy = tloggProxy
        self._notepad = NotePad()

        super().__init__(*args, **kwargs)

        self._tloggProxy.setOpenFile(self.openFile)

        self.addWidget(mainSplitter := TTkSplitter(orientation=TTkK.VERTICAL))

        menuFrame = TTkFrame(border=False, maxHeight=1)

        self._kodeTab      = TTkKodeTab(border=False, closable=True)

        fileMenu = menuFrame.newMenubarTop().addMenu("&File")
        buttonOpen    = fileMenu.addMenu("Open")
        buttonClose   = fileMenu.addMenu("Close")
        fileMenu.addSpacer()
        buttonColors  = fileMenu.addMenu("&Colors...")
        buttonFilters = fileMenu.addMenu("&Filters...")
        buttonOptions = fileMenu.addMenu("&Options...")
        fileMenu.addSpacer()
        buttonExit    = fileMenu.addMenu("Exit")
        buttonExit.menuButtonClicked.connect(TTkHelper.quit)

        extraMenu = menuFrame.newMenubarTop().addMenu("E&xtra")
        extraMenu.addMenu("Scratchpad").menuButtonClicked.connect(self.scratchpad)

        helpMenu = menuFrame.newMenubarTop().addMenu("&Help", alignment=TTkK.RIGHT_ALIGN)
        helpMenu.addMenu("About ...").menuButtonClicked.connect(self.showAbout)
        helpMenu.addMenu("About tlogg").menuButtonClicked.connect(self.showAboutTlogg)

        def _tabChanged(_,__,widget,data):
            tloggProxy.tloggFocussed.emit(None, data)

        self._kodeTab.currentChanged.connect(_tabChanged)

        # fileTree.fileActivated.connect(lambda x: self.openFile(x.path()))

        buttonOpen.menuButtonClicked.connect(self.openFileCallback)
        buttonColors.menuButtonClicked.connect(self.showColors)
        buttonFilters.menuButtonClicked.connect(self.showFilters)
        buttonOptions.menuButtonClicked.connect(self.showOptions )

        # Defining the layout based on the plugin widget placement
        if (placement:=TloggHelper._getPluginPlacements()) == ttk.TTkK.NONE:
            # If no plugin require a specific placement
            # define a basic layout without side panels
            fileControlLayout = TTkVBoxLayout()
            fileControlLayout.addWidget(menuFrame)
            fileControlLayout.addWidget(self._kodeTab)

            mainSplitter.addWidget(fileControlLayout)
            mainSplitter.addWidget(TTkLogViewer(),3)
        else:
            fileTabSplitter = TTkSplitter(orientation=TTkK.HORIZONTAL)
            fileTabSplitterSizes = []
            fileControlLayout = TTkVBoxLayout()
            fileControlLayout.addWidget(menuFrame)

            # Add left placed plugin first in the splitter
            for mod in TloggHelper._getPlacedPlugins(ttk.TTkK.LEFT):
                if fileControlLayout:
                    fileControlLayout.addWidget(mod.widget)
                    fileTabSplitter.addItem(fileControlLayout)
                    fileControlLayout = None
                else:
                    fileTabSplitter.addWidget(mod.widget)
                fileTabSplitterSizes.append(40)

            fileTabSplitter.addWidget(self._kodeTab)
            fileTabSplitterSizes.append(None)

            # Add left placed plugins at the end in the splitter
            for mod in TloggHelper._getPlacedPlugins(ttk.TTkK.RIGHT):
                if fileControlLayout:
                    fileControlLayout.addWidget(mod.widget)
                    fileTabSplitter.addItem(fileControlLayout)
                    fileControlLayout = None
                else:
                    fileTabSplitter.addWidget(mod.widget)
                fileTabSplitterSizes.append(50)

            fileTabSplitter.setSizes(fileTabSplitterSizes)

            mainSplitter.addWidget(fileTabSplitter)
            mainSplitter.addWidget(TTkLogViewer(),3)

    @pyTTkSlot()
    def scratchpad(self):
        win = TTkWindow(
                title="Mr Scratchpad",
                size=(80,30),
                layout=self._notepad,
                flags=TTkK.WindowFlag.WindowMaximizeButtonHint|TTkK.WindowFlag.WindowCloseButtonHint)
        TTkHelper.overlay(None, win, 2, 2, toolWindow=True)

    @pyTTkSlot()
    def openFileCallback(self):
        filePicker = TTkFileDialogPicker(
                        pos = (3,3), size=(90,30),
                        caption="Open a File", path=".",
                        filter="All Files (*);;Text Files (*.txt);;Log Files (*.log)")
        filePicker.filePicked.connect(self.openFile)
        TTkHelper.overlay(btn, filePicker, 10, 2, modal=True)

    @pyTTkSlot(TTkMenuButton)
    def showColors(self, btn):
        win = TTkWindow(title="Highlighters...", size=(70,20), border=True)
        win.setLayout(highlightersFormLayout(win))
        TTkHelper.overlay(btn, win, 10,2, modal=True)

    @pyTTkSlot(TTkMenuButton)
    def showFilters(self, btn):
        win = PredefinedFiltersFormWindow(title="Predefined Filters...", size=(70,20), border=True)
        TTkHelper.overlay(btn, win, 10,2, modal=True)

    @pyTTkSlot(TTkMenuButton)
    def showOptions(self, btn):
        win = TTkWindow(title="Options...", size=(70,20), border=True)
        win.setLayout(optionsFormLayout(win))
        TTkHelper.overlay(btn, win, 10,2, modal=True)

        # tab.addTab(h
        #   @pyTTkSlot()ighlightersForm(), "-Setup-")
    @pyTTkSlot(TTkMenuButton)
    def showAbout(self, btn):
        TTkHelper.overlay(btn, TTkAbout(), 20,5)

    @pyTTkSlot(TTkMenuButton)
    def showAboutTlogg(self, btn):
        TTkHelper.overlay(btn, About(), 20,5)

    def openFile(self, file):
        # openedFiles.append(file)
        loggWidget = LoggWidget(file)
        self._kodeTab.addTab(widget=loggWidget, label=os.path.basename(file), data=file)
        self._kodeTab.setCurrentWidget(loggWidget)

def main():
    TloggCfg.pathCfg = appdirs.user_config_dir("tlogg")

    parser = argparse.ArgumentParser()
    # parser.add_argument('-f', help='Full Screen', action='store_true')
    parser.add_argument('-c', help=f'config folder (default: "{TloggCfg.pathCfg}")', default=TloggCfg.pathCfg)
    parser.add_argument('filename', type=str, nargs='*',
                    help='the filename/s')
    args = parser.parse_args()

    # TTkLog.use_default_file_logging()

    TloggCfg.pathCfg = args.c
    TTkLog.debug(f"Config Path: {TloggCfg.pathCfg}")

    TloggCfg.load()

    if 'theme' not in TloggCfg.options:
        TloggCfg.options['theme'] = 'UTF8'
    optionsLoadTheme(TloggCfg.options['theme'])

    TloggHelper._loadPlugins()

    root = TTk(
            title="tlogg",
            layout=(tlogg:=TLOGG(tloggProxy=tloggProxy)),
            sigmask=(
                TTkTerm.Sigmask.CTRL_C |
                TTkTerm.Sigmask.CTRL_Q |
                TTkTerm.Sigmask.CTRL_S |
                TTkTerm.Sigmask.CTRL_Z ))
    TloggHelper._runPlugins()

    for file in args.filename:
        tlogg.openFile(file)

    root.mainloop()


#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2022 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ['optionsLoadTheme', 'optionsFormLayout']

import copy

from . import TloggCfg, TloggGlbl

from TermTk import *

def optionsLoadTheme(theme):
    if theme == 'ASCII':
        TTkTheme.loadTheme(TTkTheme.ASCII)
    elif theme == 'UTF8':
        TTkTheme.loadTheme(TTkTheme.UTF8)
    elif theme == 'NERD':
        TTkTheme.loadTheme(TTkTheme.NERD)

def optionsFormLayout(win):
    options = copy.deepcopy(TloggCfg.options)

    retLayout    = TTkGridLayout()
    bottomLayout = TTkGridLayout()

    themesFrame = TTkFrame(title="Theme", border=True, layout=TTkVBoxLayout(), maxHeight=5, minHeight=5)
    # Themes
    themesFrame.layout().addWidget(r1 := TTkRadioButton(text="ASCII", name="theme", checked=options['theme'] == 'ASCII'))
    themesFrame.layout().addWidget(r2 := TTkRadioButton(text="UTF-8", name="theme", checked=options['theme'] == 'UTF8'))
    themesFrame.layout().addWidget(r3 := TTkRadioButton(text="Nerd",  name="theme", checked=options['theme'] == 'NERD'))

    retLayout.addWidget(themesFrame,0,0)
    retLayout.addWidget(TTkSpacer() ,1,0,1,2)

    retLayout.addItem(bottomLayout ,2,0,1,2)
    bottomLayout.addWidget(applyBtn  := TTkButton(text="Apply",  border=True, maxHeight=3),0,1)
    bottomLayout.addWidget(cancelBtn := TTkButton(text="Cancel", border=True, maxHeight=3),0,2)
    bottomLayout.addWidget(okBtn     := TTkButton(text="OK",     border=True, maxHeight=3),0,3)

    def _saveOptions():
        if r1.checkState() == TTkK.Checked: options['theme'] = 'ASCII'
        if r2.checkState() == TTkK.Checked: options['theme'] = 'UTF8'
        if r3.checkState() == TTkK.Checked: options['theme'] = 'NERD'
        TloggCfg.options = options
        TloggCfg.save(searches=False, filters=False, colors=False, options=True)
        optionsLoadTheme(options['theme'])
        TloggGlbl.refreshViews()
        TTkHelper.updateAll()

    applyBtn.clicked.connect(_saveOptions)
    okBtn.clicked.connect(_saveOptions)
    okBtn.clicked.connect(win.close)
    cancelBtn.clicked.connect(win.close)

    return retLayout



# MIT License
#
# Copyright (c) 2021 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ['FileViewer','FileViewerSearch','FileViewerArea']

import TermTk as ttk

from . import TloggCfg

class FileViewer(ttk.TTkAbstractScrollView):
    __slots__ = (
        '_fileBuffer', '_indexesMark', '_indexesSearched',
        '_selected', '_indexing', '_searchRe',
        '_selection', '_pressed'
        # Signals
        'selected', 'marked')
    def __init__(self, *args, **kwargs):
        self._indexesMark = []
        self._indexesSearched = []
        self._indexing = None
        self._selected = -1
        self._selection = None
        self._pressed = False
        self._searchRe = ""
        # Signals
        self.selected = ttk.pyTTkSignal(int)
        self.marked = ttk.pyTTkSignal(list)
        self._fileBuffer = kwargs.get('filebuffer')
        super().__init__(*args, **kwargs)
        self.viewChanged.connect(self._viewChangedHandler)
        self.setFocusPolicy(ttk.TTkK.ClickFocus)

    @ttk.pyTTkSlot()
    def _viewChangedHandler(self):
        self.update()

    def _copy(self):
        clipboard = ttk.TTkClipboard()
        lines = ttk.TTkString().join([
            self.getLine(i) for i in range(min(self._selection),max(self._selection)+1) ])
        clipboard.setText(lines)

    @ttk.pyTTkSlot(float)
    def fileIndexing(self, percentage):
        self._indexing = percentage
        self.viewChanged.emit()

    @ttk.pyTTkSlot()
    def fileIndexed(self):
        self._indexing = None
        self.viewChanged.emit()

    def markIndexes(self, indexes):
        self._indexesMark = indexes
        self.viewChanged.emit()

    def searchedIndexes(self, indexes):
        self._indexesSearched = indexes
        self.viewChanged.emit()

    def searchRe(self, searchRe):
        self._searchRe = searchRe
        self.update()

    def viewFullAreaSize(self) -> (int, int):
        w = 10+self._fileBuffer.getWidth()
        h = self._fileBuffer.getLen()
        return ( w , h )

    def viewDisplayedSize(self) -> (int, int):
        return self.size()

    def keyEvent(self, evt):
        # Enable CTRL+C = Copy
        if ( evt.type == ttk.TTkK.SpecialKey and
             evt.mod  == ttk.TTkK.ControlModifier and
             evt.key  == ttk.TTkK.Key_C ):
                self._copy()

    def mousePressEvent(self, evt):
        x,y = evt.x, evt.y
        ox,oy = self.getViewOffsets()
        index = oy+y
        self._selection = [index,index]
        self._pressed = True
        if 0 <= index < self._fileBuffer.getLen():
            if x<3:
                if index in self._indexesMark:
                    self._indexesMark.pop(self._indexesMark.index(index))
                else:
                    self._indexesMark.append(index)
                self.marked.emit(self._indexesMark)
            else:
                self._selected = index
                self.selected.emit(self._selected)
            self.update()
            return True
        return super().mousePressEvent(evt)

    def mouseDragEvent(self, evt) -> bool:
        if self._pressed and self._selection:
            w,h = self.size()
            x,y = evt.x, evt.y
            lines=self._fileBuffer.getLen()
            ox,oy = self.getViewOffsets()
            index = oy+y
            self._selection[1] = min(max(0,index),lines-1)
            if y < 0:
                self.viewMoveTo(ox, max(0,min(oy+min(y,3),lines-h)))
            elif y >= h:
                self.viewMoveTo(ox, max(0,min(oy+min(y-h,3),lines-h)))
            else:
                self.update()
        return super().mouseDragEvent(evt)

    def mouseReleaseEvent(self, evt) -> bool:
        self._pressed = False
        if self._selection and self._selection[0] != self._selection[1]:
            self._copy()
        self.update()
        return super().mouseReleaseEvent(evt)

    @ttk.pyTTkSlot(int)
    def selectAndMove(self, line):
        self._selected = line
        self._selection = [line,line]
        ox,_ = self.getViewOffsets()
        self.viewMoveTo(ox, max(0,line-self.height()//2))
        self.update()

    def getLen(self):
        return self._fileBuffer.getLen()

    def getLine(self, num) -> str:
        return self._fileBuffer.getLine(num)

    def getLineNum(self, num) -> int:
        return num

    def paintEvent(self, canvas):
        ox,oy = self.getViewOffsets()
        bufferLen = self.getLen()
        for i in range(min(self.height(),bufferLen-oy)):
            line = ttk.TTkString(self.getLine(i+oy).replace('\n','')).tab2spaces()
            lineNum = self.getLineNum(i+oy)
            if lineNum in self._indexesMark:
                symbolcolor = ttk.TTkColor.fg("#00ffff")
                numberColor = ttk.TTkColor.bg("#444444")
                symbol='❥'
            elif lineNum in self._indexesSearched:
                symbolcolor = ttk.TTkColor.fg("#ff0000")
                numberColor = ttk.TTkColor.bg("#444444")
                symbol='●'
            else:
                symbolcolor = ttk.TTkColor.fg("#0000ff")
                numberColor = ttk.TTkColor.bg("#444444")
                symbol='○'

            if i+oy == self._selected:
                selectedColor = ttk.TTkColor.bg("#008844")
                searchedColor = ttk.TTkColor.fg("#FFFF00")+ttk.TTkColor.bg("#004400")
                line = line.setColor(selectedColor)
            elif self._selection and min(self._selection) <= i+oy <= max(self._selection):
                selectedColor = ttk.TTkColor.bg("#008888")
                searchedColor = ttk.TTkColor.fg("#FFFF00")+ttk.TTkColor.bg("#004400")
                line = line.setColor(selectedColor)
            else:
                selectedColor = ttk.TTkColor.RST
                searchedColor = ttk.TTkColor.fg("#000000")+ttk.TTkColor.bg("#AAAAAA")
                # Check in the filters a matching color
                for color in TloggCfg.colors:
                    #TTkLog.debug(f"{color['pattern']} - {line}")
                    if m := line.findall(regexp=color['pattern'], ignoreCase=color['ignorecase']):
                        selectedColor = ttk.TTkColor.fg(color['fg'])+ttk.TTkColor.bg(color['bg'])
                        searchedColor = ttk.TTkColor.fg(color['bg'])+ttk.TTkColor.bg(color['fg'])
                        line = line.setColor(selectedColor)
                        break
            if self._searchRe:
                if m := line.findall(regexp=self._searchRe, ignoreCase=True):
                    for match in m:
                        line = line.setColor(searchedColor, match=match)

            # Add Line Number
            lenLineNumber = len(str(self.getLineNum(bufferLen-1)))
            lineNumber = ttk.TTkString() + numberColor + str(lineNum).rjust(lenLineNumber) + ttk.TTkColor.RST + ' '
            # Compose print line
            printLine = ttk.TTkString() + symbolcolor + symbol + ttk.TTkColor.RST + ' ' + lineNumber + line.substring(ox)
            # stupid scramble
            # printLine._text = ''.join([chr(121-(ord(l)-65)) if (65<=ord(l)<=121) else l for l in printLine._text])
            canvas.drawText(pos=(0,i), text=printLine, color=selectedColor, width=self.width(), )

        # Draw the loading banner
        if self._indexing is not None:
            canvas.drawText(pos=(0,0), text=f" [ Indexed: {int(100*self._indexing)}% ] ")

class FileViewerSearch(FileViewer):
    __slots__ = ('_indexes')
    def __init__(self, *args, **kwargs):
        self._indexes = []
        FileViewer.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'FileViewerSearch' )

    def markIndexes(self, indexes):
        self._indexesMark = indexes
        self._indexes = [i for i in sorted(set(self._indexesSearched+self._indexesMark))]
        ox,oy = self.getViewOffsets()
        self.viewMoveTo(ox,oy)
        self.update()

    def searchedIndexes(self, indexes):
        # Get the current lineSelected to be used to scroll
        # the search viewer to the similar to previous position
        lineSelected = -1
        if self._selected > -1:
            lineSelected = self._indexes[self._selected]

        self._indexesSearched = indexes
        self._indexes = [i for i in sorted(set(self._indexesSearched+self._indexesMark))]
        ox,_ = self.getViewOffsets()

        lineToMove = 0
        if lineSelected > -1:
            for i, lineNum in enumerate(self._indexes):
                if lineNum >= lineSelected:
                   if lineNum == lineSelected:
                       self._selected = i
                   else:
                       self._selected = -1
                   # Try to keep the  selected area at the center of the widget
                   lineToMove = i if i <= self.height()/2 else int(i-self.height()/2)
                   break

        self.viewMoveTo(ox, lineToMove)
        self.update()

    def viewFullAreaSize(self) -> (int, int):
        if self._indexes is None:
            w = 2+self._fileBuffer.getWidth()
            h = self._fileBuffer.getLen()
        else:
            w = 2+self._fileBuffer.getWidth(self._indexes)
            h = len(self._indexes)
        return w , h

    def mousePressEvent(self, evt):
        x,y = evt.x, evt.y
        ox,oy = self.getViewOffsets()
        index = oy+y
        self._selection = [index,index]
        self._pressed = True
        if 0 <= index < len(self._indexes):
            if x<3:
                index = self._indexes[oy+y]
                if index in self._indexesMark:
                    self._indexesMark.pop(self._indexesMark.index(index))
                else:
                    self._indexesMark.append(index)
                self.markIndexes(self._indexesMark)
                self.marked.emit(self._indexesMark)
            else:
                self._selected = index
                self.selected.emit(self._indexes[self._selected])
            self.update()
            return True
        return False

    def getLen(self):
        return len(self._indexes)

    def getLine(self, num) -> str:
        return self._fileBuffer.getLine(self._indexes[num])

    def getLineNum(self, num) -> int:
        return self._indexes[num]

class FileViewerArea(ttk.TTkAbstractScrollArea):
    __slots__ = ('_fileView')
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._name = kwargs.get('name' , 'FileViewer' )
        if 'parent' in kwargs: kwargs.pop('parent')
        self._fileView = kwargs.get('fileView')
        self.setFocusPolicy(ttk.TTkK.ClickFocus)
        self.setViewport(self._fileView)



#!/usr/bin/env python3

# MIT License
#
# Copyright (c) 2022 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this soself._ftware and associated documentation files (the "Soself._ftware"), to deal
# in the Soself._ftware without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Soself._ftware, and to permit persons to whom the Soself._ftware is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Soself._ftware.
#
# THE SOself._FTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOself._FTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOself._FTWARE.

import os

from TermTk import TTkK
from TermTk import TTkWidget, TTkCheckbox, TTkButton, TTkSpacer, TTkFileTree, TTkFileTreeWidgetItem
from TermTk import TTkVBoxLayout, TTkHBoxLayout
from TermTk import pyTTkSlot, pyTTkSignal

from . import TloggCfg

class FileTree(TTkWidget):
    __slots__ = (
        '_recentPath', '_recentPathId',
        '_controlsLayout',
        '_ftCbFollow', '_ftBtnPrev', '_ftBtnNext', '_ftBtnUp', '_fileTree',
        # Forwarded Methods
        'fileActivated')

    def __init__(self, *args, **kwargs):
        TTkWidget.__init__(self, *args, **kwargs)
        self._name = kwargs.get('name' , 'FileTree' )

        self.setLayout(TTkVBoxLayout())

        self._controlsLayout = TTkHBoxLayout()
        self.layout().addItem(self._controlsLayout)

        self._ftCbFollow = TTkCheckbox(text="Follow", maxWidth=9, checked=False)
        self._ftBtnPrev  = TTkButton(text="<",maxWidth=3, enabled=False)
        self._ftBtnNext  = TTkButton(text=">",maxWidth=3, enabled=False)
        self._ftBtnUp    = TTkButton(text="^",maxWidth=3, enabled=True)
        self._controlsLayout.addWidget(self._ftCbFollow)
        self._controlsLayout.addWidget(TTkSpacer())
        self._controlsLayout.addWidget(self._ftBtnPrev)
        self._controlsLayout.addWidget(self._ftBtnNext)
        self._controlsLayout.addWidget(self._ftBtnUp)

        self._fileTree = TTkFileTree(parent=self, path=".")

        self._recentPath = []
        self._recentPathId = -1
        self._addFolder(os.path.abspath('.'))

        self._ftBtnPrev.clicked.connect(self._openPrev)
        self._ftBtnNext.clicked.connect(self._openNext)
        self._ftBtnUp.clicked.connect(self._openUp)

        self._fileTree.folderActivated.connect(self._openFolder)

        # Forward Functions
        self.fileActivated = self._fileTree.fileActivated

    def follow(self, path):
        if self._ftCbFollow.checkState() == TTkK.Checked:
            if os.path.isfile(path):
                path, _ = os.path.split(path)
            self._fileTree.openPath(path)

    def _addFolder(self, dir):
        if not self._recentPath or dir != self._recentPath[-1]:
            self._recentPath.append(dir)
        self._recentPathId = len(self._recentPath)-1
        if self._recentPathId:
            self._ftBtnPrev.setEnabled()
        self._ftBtnNext.setDisabled()

    @pyTTkSlot(TTkFileTreeWidgetItem)
    def _openFolder(self, dir):
        self._fileTree.openPath(dir.path())
        self._addFolder(dir.path())

    @pyTTkSlot()
    def _openPrev(self):
        if self._recentPathId<=0 or self._recentPathId>=len(self._recentPath):
            self._ftBtnPrev.setDisabled()
            return
        self._recentPathId -= 1
        self._fileTree.openPath(self._recentPath[self._recentPathId])
        if self._recentPathId<=0:
            self._ftBtnPrev.setDisabled()
        self._ftBtnNext.setEnabled()

    @pyTTkSlot()
    def _openNext(self):
        if self._recentPathId<0 or self._recentPathId>=len(self._recentPath)-1:
            self._ftBtnNext.setDisabled()
            return
        self._recentPathId += 1
        self._fileTree.openPath(self._recentPath[self._recentPathId])
        if self._recentPathId>=len(self._recentPath)-1:
            self._ftBtnNext.setDisabled()
        self._ftBtnPrev.setEnabled()

    @pyTTkSlot()
    def _openUp(self):
        path = os.path.abspath(self._fileTree.getOpenPath())
        path, e = os.path.split(path)
        if e:
            self._addFolder(path)
            self._fileTree.openPath(path)



# MIT License
#
# Copyright (c) 2023 Eugenio Parodi <ceccopierangiolieugenio AT googlemail DOT com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

__all__ = ['LoggWidget']

import TermTk as ttk

from .cfg  import TloggCfg
from .glbl import TloggGlbl
from .fileviewer  import FileViewer, FileViewerArea, FileViewerSearch
from .predefinedfilters import PredefinedFilters

class LoggWidget(ttk.TTkSplitter):
    __slots__ = ('_btn_filters', '_bls_label_1', '_bls_cb_icase', '_bls_search', '_bls_searchbox',
                 '_topViewport', '_bottomViewport',
                 '_fileBuffer')
    def __init__(self, filename, *args, **kwargs):
        super().__init__(*args, **kwargs|{'orientation':ttk.TTkK.VERTICAL})

        topFrame    = ttk.TTkFrame(parent=self, border=False, layout=ttk.TTkVBoxLayout())
        bottomFrame = ttk.TTkFrame(parent=self, border=False, layout=ttk.TTkVBoxLayout())


        # Define the bottom layout widgets
        bottomLayoutSearch = ttk.TTkHBoxLayout()
        self._btn_filters   = ttk.TTkButton(text="Filters ^", maxWidth=11)
        self._bls_label_1   = ttk.TTkLabel(text=" Txt:", maxWidth=5)
        self._bls_cb_icase  = ttk.TTkCheckbox(text="Aa", maxWidth=5, checked=True)
        self._bls_search    = ttk.TTkButton(text="Search", maxWidth=10)
        self._bls_searchbox = ttk.TTkComboBox(editable=True)
        self._bls_searchbox.addItems(TloggCfg.searches)
        self._bls_searchbox.setCurrentIndex(0)

        bottomLayoutSearch.addWidget(self._btn_filters)
        bottomLayoutSearch.addWidget(self._bls_label_1)
        bottomLayoutSearch.addWidget(self._bls_searchbox)
        bottomLayoutSearch.addWidget(self._bls_cb_icase)
        bottomLayoutSearch.addWidget(self._bls_search)

        bottomFrame.layout().addItem(bottomLayoutSearch)

        # Define the main file Viewer
        self._fileBuffer = ttk.TTkFileBuffer(filename, 0x100, 0x1000)
        self._topViewport = FileViewer(filebuffer=self._fileBuffer)
        topViewer = FileViewerArea(parent=topFrame, fileView=self._topViewport)
        self._fileBuffer.indexUpdated.connect(self._topViewport.fileIndexing)
        self._fileBuffer.indexed.connect(self._topViewport.fileIndexed)
        # Define the Search Viewer
        self._bottomViewport = FileViewerSearch(filebuffer=self._fileBuffer)
        bottomViewer = FileViewerArea(parent=bottomFrame, fileView=self._bottomViewport)
        self._bottomViewport.selected.connect(self._topViewport.selectAndMove)
        self._bottomViewport.marked.connect(self._topViewport.markIndexes)
        self._topViewport.marked.connect(self._bottomViewport.markIndexes)

        # Add those viewpoers to the global list to allow dynamic refresh
        # TODO: Try to get rid of this
        TloggGlbl.addRefView(self._topViewport)
        TloggGlbl.addRefView(self._bottomViewport)

        self._bls_search.clicked.connect(self._search)
        self._bls_searchbox.editTextChanged.connect(self._search)

        def _openPredefinedFilters():
            ttk.TTkHelper.overlay(self._btn_filters, PredefinedFilters(self._bls_searchbox), -2, 1)
        self._btn_filters.clicked.connect(_openPredefinedFilters)

    @ttk.pyTTkSlot()
    def _search(self):
        searchtext = str(self._bls_searchbox.currentText())
        ttk.TTkLog.debug(f"{searchtext=}")
        indexes = self._fileBuffer.searchRe(searchtext, ignoreCase=self._bls_cb_icase.checkState() == ttk.TTkK.Checked)
        self._bottomViewport.searchedIndexes(indexes)
        self._bottomViewport.searchRe(searchtext)
        self._topViewport.searchedIndexes(indexes)
        self._topViewport.searchRe(searchtext)
        if TloggCfg.searches:
            x = set(TloggCfg.searches)
            ttk.TTkLog.debug(f"{x}")
            TloggCfg.searches = list(x)
            if searchtext in TloggCfg.searches:
                TloggCfg.searches.remove(searchtext)
        TloggCfg.searches.insert(0, searchtext)
        TloggCfg.searches = TloggCfg.searches[:TloggCfg.maxsearches]
        TloggCfg.save(searches=True,filters=False)
        self._bls_searchbox.clear()
        self._bls_searchbox.addItems(TloggCfg.searches)
        self._bls_searchbox.setCurrentIndex(0)
